@@ -41,6 +41,7 @@ from . import cache
 from . import compress
 from . import config
 from . import controller
+from . import data
 from . import defines
 from . import exceptions
 from . import export
@@ -69,11 +70,12 @@ from .api import Api, OAuthApi, OAuth1Api, OAuth2Api
 from .async import AsyncManager, SimpleManager, QueueManager
 from .base import APP, LEVEL, NAME, VERSION, PLATFORM, API_VERSION, BUFFER_SIZE, MAX_LOG_SIZE,\
     MAX_LOG_COUNT, App, APIApp, WebApp, get_app, get_name, get_base_path, get_request,\
-    get_session, get_model, get_controller, get_level, is_devel
+    get_session, get_model, get_controller, get_adapter, get_level, is_devel, is_safe
 from .cache import Cache, MemoryCache
 from .compress import Compress
 from .config import conf, conf_prefix, conf_s
 from .controller import Controller
+from .data import DataAdapter, MongoAdapter, Collection, MongoCollection
 from .defines import ITERABLES, MOBILE_REGEX, MOBILE_PREFIX_REGEX, BODY_REGEX, TAG_REGEX,\
     EMAIL_REGEX, WINDOWS_LOCALE
 from .exceptions import AppierException, OperationalError, SecurityError, ValidationError,\
@@ -84,7 +86,7 @@ from .git import Git
 from .http import get, post, put, delete
 from .log import MemoryHandler, ThreadFormatter, rotating_handler, smtp_handler, in_signature
 from .meta import Ordered, Indexed
-from .model import Model, Field, link, operation, field
+from .model import Model, LocalModel, Field, link, operation, field
 from .mongo import Mongo, get_connection, get_db, drop_db, object_id, dumps
 from .observer import Observable
 from .part import Part
@@ -107,8 +109,8 @@ from .util import is_iterable, is_mobile, email_parts, email_mime, email_name, e
     sanitize, execute, FileTuple, BaseThread
 from .validation import validate, validate_b, safe, eq, gt, gte, lt, lte, not_null, not_empty,\
     not_false, is_in, is_simple, is_email, is_url, is_regex, field_eq, field_gt, field_gte,\
-    field_lt, field_lte, string_gt, string_lt, equals, not_past, not_duplicate, all_different,\
-    no_self
+    field_lt, field_lte, string_gt, string_lt, string_eq, equals, not_past, not_duplicate,\
+    all_different, no_self
 
 from .mongo import get_connection as get_mongo
 from .mongo import get_db as get_mongo_db
