@@ -78,6 +78,15 @@ custom builder functions to be used when applying
 the types function, this is relevant for the built-in
 types that are meant to avoid using the default constructor """
 
+BUILDERS_META = dict(
+    country = BUILDERS[legacy.UNICODE],
+    longtext = BUILDERS[legacy.UNICODE],
+    file = None
+)
+""" Map equivalent to the builders map but appliable
+for meta based type naming, this map should be used as
+an extension to the base builder map """
+
 METAS = dict(
     text = lambda v, d: v,
     enum = lambda v, d: d["enum"].get(v, None),
@@ -206,6 +215,8 @@ INSENSITIVE = {
 values that define if an insensitive base search should be used
 instead of the "typical" sensitive search """
 
+BUILDERS.update(BUILDERS_META)
+
 class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
     """
     Abstract model class from which all the models should
@@ -223,11 +234,13 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         instance.__dict__["_extras"] = []
         instance.__dict__["model"] = {}
         instance.__dict__["owner"] = None
+        instance.__dict__["ref"] = None
         return instance
 
     def __init__(self, model = None, **kwargs):
         self.__dict__["model"] = model or {}
         self.__dict__["owner"] = common.base().APP or None
+        self.__dict__["ref"] = kwargs.pop("ref", None)
         for name, value in kwargs.items(): setattr(self, name, value)
         observer.Observable.__init__(self)
 
@@ -369,7 +382,7 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         )
 
     @classmethod
-    def wrap(cls, models, build = True, handler = None):
+    def wrap(cls, models, build = True, handler = None, **kwargs):
         """
         "Wraps" the provided sequence (or single set) of model based data into a
         sequence of models (or a single model) so that proper business logic may
@@ -379,6 +392,9 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         each model that is going to be "wrapped" allowing an extra "layer" for
         the transformation of the model.
 
+        The additional named arguments parameters allows extensibility to set
+        extra value in the creation of the wrapped object.
+
         This operation is specially useful for api based environments where client
         side business logic is meant to be added to the static data.
 
@@ -402,9 +418,9 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         if not is_sequence: models = [models]
         wrapping = []
         for model in models:
-            _model = cls(model = model)
-            build and cls.build(_model.model, map = False)
+            _model = cls(model = model, **kwargs)
             handler and handler(_model.model)
+            build and cls.build(_model.model, map = False)
             wrapping.append(_model)
         if is_sequence: return wrapping
         else: return wrapping[0]
@@ -442,7 +458,7 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
             ("build", True),
             ("fill", True),
             ("skip", 0),
-            ("limit", 10),
+            ("limit", 0),
             ("sort", None),
             ("raise_e", True)
         ))
@@ -451,13 +467,7 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         collection = cls._collection()
         model = collection.find_one(
             kwargs,
-            projection = fields,
-            skip = skip,
-            limit = limit,
-            sort = sort
-        ) if mongo.is_new() else collection.find_one(
-            kwargs,
-            fields = fields,
+            fields,
             skip = skip,
             limit = limit,
             sort = sort
@@ -488,7 +498,7 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
             ("build", True),
             ("fill", True),
             ("skip", 0),
-            ("limit", 10),
+            ("limit", 0),
             ("sort", None)
         ))
 
@@ -499,13 +509,7 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         collection = cls._collection()
         models = collection.find(
             kwargs,
-            projection = fields,
-            skip = skip,
-            limit = limit,
-            sort = sort
-        ) if mongo.is_new() else collection.find(
-            kwargs,
-            fields = fields,
+            fields,
             skip = skip,
             limit = limit,
             sort = sort
@@ -547,6 +551,12 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         index = math.floor(index)
         index = int(index) + 1
 
+        # calculates the proper size of the current page being requested
+        # taking into account the total number of values and the limit
+        size = total % limit if index == count else limit
+        if size == 0 and total > 0: size = limit
+        if total == 0: size = 0
+
         # creates the base structure for the page populating with the
         # base values that may be used for display of the page
         page = dict(
@@ -554,6 +564,7 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
             index = index,
             start = skip + 1,
             end = skip + limit,
+            size = size,
             total = total,
             sorter = request.params_s.get("sorter", None),
             direction = request.params_s.get("direction", "descending")
@@ -1201,6 +1212,10 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         # based arguments map for the currently defined name
         kwargs[name] = filter
 
+    @classmethod
+    def is_attached(self):
+        return True
+
     @classmethod
     def _build(cls, model, map):
         pass
@@ -1236,10 +1251,10 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         return definition.get("meta", base)
 
     @classmethod
-    def _collection(cls):
-        name = cls._name()
-        db = mongo.get_db()
-        collection = db[name]
+    def _collection(cls, name = None):
+        name = name or cls._name()
+        adapter = common.base().get_adapter()
+        collection = adapter.collection(name)
         return collection
 
     @classmethod
@@ -1434,7 +1449,7 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
             cls.filter_merge(name, find_v, kwargs)
 
     @classmethod
-    def _bases(cls):
+    def _bases(cls, subclass = None):
         """
         Retrieves the complete set of base (parent) classes for
         the current class, this method is safe as it removes any
@@ -1443,16 +1458,28 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         Please note that this function only retrieves the set of
         direct parents of the class and not the complete hierarchy.
 
+        :type subclass: Class
+        :param subclass: The subclass from which the various base
+        classes must inherit to be considered model, in case the
+        value is not defined the "basic" model value is used.
         :rtype: List/Tuple
         :return: The set containing the various bases classes for
         the current class that are considered valid.
         """
 
+        # determines the base sub class value that is going
+        # to be applied for proper sub class validation
+        subclass = subclass or Model
+
         # retrieves the complete set of base classes for
         # the current class and in case the observable is
         # not in the bases set returns the set immediately
-        # as the top level model has not been reached yet
+        # as the top level model has not been reached yet,
+        # note that the various base classes are filtered
+        # according to their inheritance from the provided
+        # (model) class to be applied as filter
         bases = cls.__bases__
+        if subclass: bases = [base for base in bases if issubclass(base, subclass)]
         if not bases == Model.__bases__: return bases
 
         # returns an empty tuple to the caller method as the
@@ -1463,12 +1490,12 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
     @classmethod
     def _increment(cls, name):
         _name = cls._name() + ":" + name
-        db = mongo.get_db()
-        value = db.counters.find_and_modify(
-            query = {
+        store = cls._collection(name = "counters")
+        value = store.find_and_modify(
+            {
                 "_id" : _name
             },
-            update = {
+            {
                 "$inc" : {
                     "seq" : 1
                 }
@@ -1476,7 +1503,7 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
             new = True,
             upsert = True
         )
-        value = value or db.counters.find_one({
+        value = value or store.find_one({
             "_id" : _name
         })
         return value["seq"]
@@ -1668,6 +1695,20 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         # delete operation, this should trigger changes in the model
         self.post_delete()
 
+    def approve(self, model = None, type = None):
+        # retrieves the class associated with the instance
+        # that is going to be sent for approval
+        cls = self.__class__
+
+        # determines the proper method naming suffix to be
+        # used for the requested type of approval/validation
+        suffix = "_" + type if type else ""
+
+        # retrieves the proper (target) method for validation
+        # and then runs the inner validate method for it
+        method = getattr(cls, "validate" + suffix)
+        self._validate(model = model, method = method)
+
     def reload(self, *args, **kwargs):
         is_new = self.is_new()
         if is_new: raise exceptions.OperationalError(
@@ -1677,13 +1718,18 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         cls = self.__class__
         return cls.get(_id = self._id, *args, **kwargs)
 
-    def map(self):
-        model = self._filter()
+    def map(self, resolve = False, all = False):
+        model = self._filter(resolve = resolve, all = all)
         return model
 
     def dumps(self):
         return mongo.dumps(self.model)
 
+    def unwrap(self, **kwargs):
+        default = kwargs.get("default", False)
+        if default: return dict(self.model)
+        else: return dict()
+
     def pre_validate(self):
         self.trigger("pre_validate")
 
@@ -1726,7 +1772,7 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
     def _delete(self):
         pass
 
-    def _validate(self, model = None):
+    def _validate(self, model = None, method = None):
         # calls the event handler for the validation process this
         # should setup the operations for a correct validation
         self.pre_validate()
@@ -1742,8 +1788,8 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         # checks if the current model is new (create operation)
         # and sets the proper validation methods retrieval method
         is_new = self.is_new()
-        if is_new: method = cls.validate_new
-        else: method = cls.validate
+        if is_new: method = method or cls.validate_new
+        else: method = method or cls.validate
 
         # runs the validation process on the various arguments
         # provided to the account and in case an error is returned
@@ -1786,7 +1832,14 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         # should finish the operations from a correct validation
         self.post_validate()
 
-    def _filter(self, increment_a = True, immutables_a = False, normalize = False):
+    def _filter(
+        self,
+        increment_a = True,
+        immutables_a = False,
+        normalize = False,
+        resolve = False,
+        all = False
+    ):
         # creates the model that will hold the "filtered" model
         # with all the items that conform with the class specification
         model = {}
@@ -1834,6 +1887,25 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
                 if not hasattr(value, "ref_v"): continue
                 model[name] = value.ref_v()
 
+        # in case the resolution flag is set, it means that a recursive
+        # approach must be performed for the resolution of values that
+        # implement the map value (recursive resolution) method, this is
+        # a complex (and possible computational expensive) process that
+        # may imply access to the base data source
+        if resolve:
+            for name, value in legacy.eager(self.model.items()):
+                if not name in definition: continue
+                model[name] = self._resolve(name, value)
+
+        # in case the all flag is set the extra fields (not present
+        # in definition) must also be used to populate the resulting
+        # (filtered) map so that it contains the complete set of values
+        # present in the base map of the current instance
+        if all:
+            for name, value in legacy.eager(self.model.items()):
+                if name in model: continue
+                model[name] = value
+
         # returns the model containing the "filtered" items resulting
         # from the validation of the items against the model class
         return model
@@ -1862,6 +1934,34 @@ class Model(legacy.with_meta(meta.Ordered, observer.Observable)):
         value = value.json_v() if hasattr(value, "json_v") else value
         return value
 
+    def _resolve(self, name, value, all = False):
+        # verifies if the current value is an iterable one in case
+        # it is runs the evaluate method for each of the values to
+        # try to resolve them into the proper representation
+        is_iterable = hasattr(value, "__iter__")
+        is_iterable = is_iterable and not type(value) in ITERABLES
+        if is_iterable: return [self._resolve(name, value) for value in value]
+
+        # verifies if the map value recursive approach should be used
+        # for the element and if that's the case calls the proper method
+        # otherwise uses the provided (raw value)
+        if not hasattr(value, "map_v"): return value
+        return value.map_v(resolve = True, all = all)
+
+class LocalModel(Model):
+    """
+    Concrete model aimed at cases where data source based
+    persistence is not required, while the remainder model
+    features are useful.
+
+    Typical uses cases involve API based validation or local
+    transient model usage.
+    """
+
+    @classmethod
+    def is_attached(self):
+        return False
+
 class Field(dict):
     """
     Top level field class that should be used for the
@@ -1929,8 +2029,7 @@ class Operation(dict):
         for value, parameters in zip(values, parameters):
             cast = parameters[2]
             is_default = value in (None, "")
-            if cast in ("file",): cast = None
-            if cast in ("longtext",): cast = legacy.UNICODE
+            cast = BUILDERS.get(cast, cast)
             if cast and not is_default: value = cast(value)
             if is_default: value = TYPE_DEFAULTS.get(cast, value)
             casted.append(value)
