@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Hive Appier Framework
-# Copyright (C) 2008-2014 Hive Solutions Lda.
+# Copyright (c) 2008-2015 Hive Solutions Lda.
 #
 # This file is part of Hive Appier Framework.
 #
@@ -31,7 +31,7 @@ __revision__ = "$LastChangedRevision$"
 __date__ = "$LastChangedDate$"
 """ The last change date of the module """
 
-__copyright__ = "Copyright (c) 2008-2014 Hive Solutions Lda."
+__copyright__ = "Copyright (c) 2008-2015 Hive Solutions Lda."
 """ The copyright for the module """
 
 __license__ = "Apache License, Version 2.0"
@@ -39,9 +39,10 @@ __license__ = "Apache License, Version 2.0"
 
 import json
 
-from appier import typesf
-from appier import config
-from appier import common
+from . import legacy
+from . import typesf
+from . import config
+from . import common
 
 try: import pymongo
 except: pymongo = None
@@ -62,7 +63,10 @@ class Mongo(object):
     def get_connection(self):
         if self._connection: return self._connection
         url = config.conf("MONGOHQ_URL", "mongodb://localhost:27017")
-        self._connection = pymongo.Connection(url)
+        url = config.conf("MONGOLAB_URI", url)
+        url = config.conf("MONGO_URL", url)
+        if is_new(): self._connection = pymongo.MongoClient(url)
+        else: self._connection = pymongo.Connection(url)
         return self._connection
 
     def get_db(self, name):
@@ -75,10 +79,19 @@ def get_connection():
     global connection
     if connection: return connection
     url = config.conf("MONGOHQ_URL", "mongodb://localhost:27017")
-    connection = pymongo.Connection(url)
+    url = config.conf("MONGOLAB_URI", url)
+    url = config.conf("MONGO_URL", url)
+    if is_new(): connection = pymongo.MongoClient(url)
+    else: connection = pymongo.Connection(url)
     return connection
 
 def get_db(name = None):
+    url = config.conf("MONGOHQ_URL", None)
+    url = config.conf("MONGOLAB_URI", url)
+    url = config.conf("MONGO_URL", url)
+    result = legacy.urlparse(url or "")
+    name = result.path.strip("/") if result.path else None
+    name = name or config.conf("MONGO_DB", name)
     name = name or common.base().get_name() or "master"
     connection = get_connection()
     db = connection[name]
@@ -106,3 +119,22 @@ def is_mongo(obj):
     if bson and isinstance(obj, bson.ObjectId): return True
     if bson and isinstance(obj, bson.DBRef): return True
     return False
+
+def is_new():
+    return int(pymongo.version[0]) >= 3 if pymongo else False
+
+def _store_find_and_modify(store, *args, **kwargs):
+    if is_new(): store.find_one_and_update(*args, **kwargs)
+    else: store.find_and_modify(*args, **kwargs)
+
+def _store_insert(store, *args, **kwargs):
+    if is_new(): store.insert_one(*args, **kwargs)
+    else: store.insert(*args, **kwargs)
+
+def _store_update(store, *args, **kwargs):
+    if is_new(): store.update_one(*args, **kwargs)
+    else: store.update(*args, **kwargs)
+
+def _store_ensure_index(store, *args, **kwargs):
+    if is_new(): store.create_index(*args, **kwargs)
+    else: store.ensure_index(*args, **kwargs)
