@@ -102,6 +102,7 @@ ALIAS = {
     "filter_string" : "find_s",
     "insensitive" : "find_i",
     "order" : "sort",
+    "offset" : "skip",
     "start_record" : "skip",
     "number_records" : "limit"
 }
@@ -120,6 +121,12 @@ FIND_TYPES = dict(
 """ The map associating the various find fields with
 their respective types """
 
+FIND_DEFAULTS = dict(
+    limit = 10
+)
+""" The map that defines the various default values
+for a series of find related attributes """
+
 def is_iterable(object):
     """
     Verifies if the provided object (value) is iterable
@@ -236,7 +243,14 @@ def request_json(request = None, encoding = "utf-8"):
     # may be used as the parsed value (post information)
     return data_j
 
-def get_object(object = None, alias = False, page = False, find = False, norm = True):
+def get_object(
+    object = None,
+    alias = False,
+    page = False,
+    find = False,
+    norm = True,
+    **kwargs
+):
     # retrieves the base request object that is going to be used in
     # the construction of the object
     request = common.base().get_request()
@@ -265,6 +279,7 @@ def get_object(object = None, alias = False, page = False, find = False, norm =
     alias and resolve_alias(object)
     page and page_types(object)
     find and find_types(object)
+    find and find_defaults(object, kwargs)
 
     # in case the normalization flag is set runs the normalization
     # of the provided object so that sequences are properly handled
@@ -302,6 +317,16 @@ def find_types(object):
         find_type = FIND_TYPES[name]
         object[name] = find_type(value)
 
+def find_defaults(object, kwargs):
+    for name, value in legacy.iteritems(kwargs):
+        if name in object: continue
+        if not name in FIND_TYPES: continue
+        object[name] = value
+
+    for name, value in legacy.iteritems(FIND_DEFAULTS):
+        if name in object: continue
+        object[name] = value
+
 def norm_object(object):
     # iterates over all the key value association in the
     # object, trying to find the ones that refer sequences
@@ -456,6 +481,7 @@ def gen_token():
     """
 
     token_s = str(uuid.uuid4())
+    token_s = token_s.encode("utf-8")
     token = hashlib.sha256(token_s).hexdigest()
     return token
 
