@@ -42,7 +42,6 @@ import copy
 import datetime
 
 from . import util
-from . import mongo
 from . import common
 from . import legacy
 from . import exceptions
@@ -306,7 +305,15 @@ def string_lt(name, size, message = "must be smaller than %d characters"):
         raise exceptions.ValidationInternalError(name, message % size)
     return validation
 
-def equals(first_name, second_name, message = "value is not equals to %s"):
+def string_eq(name, size, message = "must be exactly %d characters"):
+    def validation(object, ctx):
+        value = object.get(name, None)
+        if value == None: return True
+        if len(value) == size: return True
+        raise exceptions.ValidationInternalError(name, message % size)
+    return validation
+
+def equals(first_name, second_name, message = "value is not equal to %s"):
     def validation(object, ctx):
         first_value = object.get(first_name, None)
         second_value = object.get(second_name, None)
@@ -330,8 +337,8 @@ def not_duplicate(name, collection, message = "value is duplicate"):
         value = object.get(name, None)
         if value == None: return True
         if value == "": return True
-        db = mongo.get_db()
-        _collection = db[collection]
+        adapter = common.base().get_adapter()
+        _collection = adapter.collection(collection)
         item = _collection.find_one({name : value})
         if not item: return True
         if str(item["_id"]) == str(_id): return True
